#!/bin/ksh
#
#	$Id: release.ksh,v 1.12 2015/02/02 18:58:45 craski-shell_86 Exp $
#
# vim: tabstop=4 shiftwidth=4 softtabstop=4 noexpandtab
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
# Copyright (c) 2014-2015 Craig R. Skinner <skinner@britvault.co.uk>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#

#
# This script is used by the maintainer to generate new releases.
#


. /usr/local/lib/sysexits.sh


PROJECT=${PWD##*/}
VERSION=$(< VERSION)
CVS_TAG="V_$(print ${VERSION} | tr '.' '_')"
FULLPKGNAME="${PROJECT}-${VERSION}"
DISTRIBUTION="dist/${FULLPKGNAME}"

# ports builtin variables:
SYSCONFDIR='/etc'
LOCALBASE='/usr/local'
VARBASE='/var'
BASELOCALSTATEDIR="${VARBASE}"

# port Makefile variables:
INST_DIR='sbin'
DOC_DIR="share/doc/${PROJECT}"
LOCALSTATEDIR="${BASELOCALSTATEDIR}/db"
DAEMON='spamd'
DAEMON_DB_TOOL="${DAEMON}b"
DAEMON_DB="${LOCALSTATEDIR}/${DAEMON}"
COMMENT="pf ${DAEMON} database assistant"
TRAP_ADDRESSES="${SYSCONFDIR}/mail/${DAEMON}.trap-addresses"
MAN_SECTION='8'
MAN_TITLE="Postmaster's Manual"

SUBST_VARS='COMMENT \
			PROJECT \
			VERSION \
			INST_DIR \
			DOC_DIR \
			DAEMON \
			DAEMON_DB \
			DAEMON_DB_TOOL \
			TRAP_ADDRESSES \
			MAN_SECTION \
			MAN_TITLE \
			\
			VARBASE \
			LOCALSTATEDIR \
			SYSCONFDIR \
			LOCALBASE \
			FULLPKGNAME'


function prepare
{
	rm -rf ${DISTRIBUTION}*
	install -d ${DISTRIBUTION} || exit
}


function build
{
	whence pod2man pod2html > /dev/null || exit

	substitute_variables README.tmpl
	substitute_variables ${PROJECT}.${SHELL##*/}

	substitute_variables ${PROJECT}.pod.tmpl
	pod2man -s ${MAN_SECTION} -c "${MAN_TITLE}" ${PROJECT}.pod ${PROJECT}.${MAN_SECTION}
	cvs diff ${PROJECT}.${MAN_SECTION} > /dev/null || cvs commit '' ${PROJECT}.${MAN_SECTION}

	pod2html ${PROJECT}.pod --outfile=${PROJECT}.html
	cvs diff ${PROJECT}.html > /dev/null || cvs commit '' ${PROJECT}.html
}


function substitute_variables
{
	file=$1
	[[ -f ${file} ]] || exit ${EX_NOINPUT}
	sed 's/\$Id\: /\$SRC_Id\: /' ${file} > ${file%.*}

	for var in $(print ${SUBST_VARS} | tr -d \\)
	do
		fgrep -q \${${var}} ${file%.*} || continue
		print "H\n,s|\${${var}}|$(eval print \${${var}})|g\n.\nw" |
			ed -s ${file%.*} >/dev/null || exit ${EX_SOFTWARE}
	done

	export $(stat -s ${file})
	chmod ${st_mode} ${file%.*}

	cvs diff -I '\$SRC_Id.*\$$' ${file%.*} > /dev/null || cvs commit '' ${file%.*}
}


function cvs_tag
{
	changelog

	cvs status | grep '^File: ' | grep -v 'Status: Up-to-date$' &&
	{
		print -u2 'cvs commit the above before preceeding.'
		exit ${EX_DATAERR}
	}
	cvs commit ${CVS_TAG} ChangeLog
	cvs tag ${CVS_TAG}
}


function changelog
{
	whence cvs2cl > /dev/null || exit

	cvs2cl --revisions --tagdates --no-wrap \
	--gecos --day-of-week --prune \
	--ignore ChangeLog --stdout | sed 's/): /):\
	/' > ChangeLog
	cvs diff ChangeLog >/dev/null || cvs commit '' ChangeLog
}


function cvs_export
{
	cvs export -r ${CVS_TAG} -d ${DISTRIBUTION} $(< CVS/Repository) || exit
}


function archive
{
	rm -f ${DISTRIBUTION}.tar.gz
	cd ${DISTRIBUTION%/*} || exit
	tar cvzf ${DISTRIBUTION##*/}.tar.gz ${DISTRIBUTION##*/} || exit
	cd ${OLDPWD}
}


umask 022


case ${1} in
	prepare|build|cvs_tag|changelog|cvs_export|archive)
		$1
		;;
	all)
		prepare
		build
		cvs_tag
		cvs_export
		archive
		;;
	*)
		print -u2 "Usage: $0 (prepare|build|cvs_tag|changelog|cvs_export|archive|all)"
		exit ${EX_USAGE}
		;;
esac

